from django.test import TestCase
from ..forms import MediaField, MediaFieldWidget


class TestMediaFormField(TestCase):
    def test_media_field_inherits_from_image_field(self):
        """MediaField should inherit from standard ImageField.
        """
        from django.forms import ImageField
        self.assertIn(ImageField, MediaField.__bases__)

    def test_media_field_should_have_correct_widget(self):
        """MediaField of a form should have MediaFieldWidget.
        """
        self.assertEqual(MediaField.widget, MediaFieldWidget)


class TestMediaFieldWidget(TestCase):
    def test_widget_renders_correct_string(self):
        """test_widget_renders_correct_string.
        """
        widget = MediaFieldWidget()
        self.assertEqual(widget.render('hello', 'world'), 'hello world!!!')



from django.db.models import ImageField
from .forms import MediaField as MF
from django.core.files.base import ContentFile
from django.core.files.uploadedfile import InMemoryUploadedFile
import urllib2
from django.core.exceptions import SuspiciousFileOperation


class MediaField(ImageField):
    def formfield(self, **kwargs):
        defaults = {'form_class': MF}
        defaults.update(kwargs)
        return super(ImageField, self).formfield(**defaults)

    def save_form_data(self, instance, data):
        # Important: None means "no change", other false value means "clear"
        # This subtle distinction (rather than a more explicit marker) is
        # needed because we need to consume values that are also sane for a
        # regular (non Model-) Form to find in its cleaned_data dictionary.
        # To default None for "no change" empty string '' also added, because
        # all changes to the field are made through input.hidden. The latter
        # is not initialized when the form is rendered for already existing
        # model.
        if data is not None and data is not u'':
            # This value will be converted to unicode and stored in the
            # database, so leaving False as-is is not acceptable.
            if not data:
                data = ''
            else:
                file_name = data.split('/')[-1]
                resource = urllib2.urlopen(data)
                if 'image' != resource.headers.getmaintype():
                    raise SuspiciousFileOperation('You are trying to download\
                                                  not image')
                f = ContentFile(resource.read())
                data = InMemoryUploadedFile(
                    f,
                    self.name,
                    file_name,
                    resource.headers.gettype(),
                    f.size,
                    resource.headers.getencoding()
                )
                self._committed = False

                # delete current file associated with the field
                field = getattr(instance, self.name, None)
                if field:
                    field.delete(False)
            setattr(instance, self.name, data)



from setuptools import setup, find_packages

setup(
  name = 'django-adv-imagefield',
  packages=find_packages(),
  include_package_data=True,
  version = '0.0.4',
  description = 'Advanced ImageField for Django that provides widget to search Flickr and Google Images',
  author = 'Devang Mundhra',
  author_email = 'devangmundhra@gmail.com',
  url = 'https://github.com/devangmundhra/django-adv-imagefield',
  license='MIT',
  keywords = ['django', 'imagefield', 'media', 'flickr', 'google-image-search',],
  install_requires=[
        'Django>=1.7',
        'Pillow>=2.7.0',
        'beautifulsoup4>=4.3.2',
        ],
  classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Utilities',
    ],
)


from django.http import JsonResponse
import urllib2
from bs4 import BeautifulSoup
import re


def url_image_list(request):
    if request.method == 'GET':
        url = request.GET.get('url', None)
        response = {'status': 'error', 'urls': []}

        # Working only with http and https.
        # There is a check to using one of these protocols.
        # If a user defined URL without protocol prefix -- add it.
        # If a user defined another protocol, ValueError exception will be
        # raised below.
        if url:
            http = re.search(r'^http://', url)
            https = re.search(r'^https://', url)
            if not http and not https:
                url = 'http://' + url
        else:
            return JsonResponse(response)

        try:
            f = urllib2.urlopen(url, None, 10)
            mime_type = f.headers.gettype()

            # URL points directly to image
            if 'image' in mime_type:
                response['urls'] = [{'url': url}]
                response['status'] = 'ok'
            # URL points to human-readable text and source code
            elif mime_type == 'text/html' or mime_type == 'text/xml':
                url_content = f.read()
                soup = BeautifulSoup(url_content)
                imgs = soup.findAll('img')
                url_set = set([img['src'] for img in imgs])
                urls = [{'url': url_item} for url_item in url_set]
                response['urls'] = urls
                response['status'] = 'ok'
        except ValueError:
            response = {'status': 'error'}
        except urllib2.URLError:
            response = {'status': 'error'}
        return JsonResponse(response)



from django.test import TestCase
from django.db.models import ImageField

from media_field.db import MediaField
from media_field import forms


class TestMediaModelField(TestCase):
    def test_media_field_inherits_from_image_field(self):
        """MediaField should inherit from standard ImageField.
        """
        self.assertIn(ImageField, MediaField.__bases__)

    def test_media_field_referres_the_correct_form_field(self):
        """MediaField of a model should point to MediaField for forms. That
        will be used while utilizing ModelForms.
        """
        media_field = MediaField()
        self.assertTrue(isinstance(media_field.formfield(), forms.MediaField))



from django.forms import URLField
from django.forms.widgets import Widget
from django.utils.html import format_html
from django.forms.utils import flatatt
from django.db.models.fields.files import ImageFieldFile
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse


widget_template = '''
<div {attrs} class="media_widget_wrapper" {width}>
    <div class="media_widget_row">
        <img id="media_widget_image" src="{value}" {image_width}/>
        <input id="media_widget_image_hidden" type="hidden" name="{name}">
        </input>
    </div>
    <br/>
    <div class="media_widget_row">
        <div class="media_widget_inputs" {image_width}>
            <input id="searchInput" placeholder="Enter Search Term" type="text">
            </input>
            <input id="searchButton" type="submit" value="Search">
            </input>
        </div>
        <br/>
    </div>
    <div class="media_widget_row" id="flickr_google_wrapper">
    </div>
    <br/>
    <div class="media_widget_row">
        <div class="media_widget_inputs" {image_width}>
            <input id="urlInput" placeholder="http://example.com" type="text">
            </input>
            <input id="urlButton" type="submit" value="Go">
            </input>
        </div>
    </div>
    <div class="media_widget_row" id="url_wrapper">
    </div>
</div>
<script type="text/javascript" charset="utf-8">
    var FLICKR_API_KEY = "{flickr_api}";
    var GOOGLE_API_KEY = "{google_api}";
    var GOOGLE_SENGINE_ID = "{google_sengine}";
    var MEDIA_FIELD_API = "{media_field_api}";
</script>'''


class MediaFieldWidget(Widget):
    class Media:
        css = {
            'all': ('/static/media_field/media_field.css',)
        }
        js = ('/static/media_field/mediaField.js',)

    def render(self, name, value, attrs=None):
        val = ''
        if type(value) is ImageFieldFile:
            val = value.url
        if not attrs:
            attrs = {}

        width_value = attrs.pop('width', None)
        width = 'style="width:{0}"'.format(width_value) if width_value else ''
        image_width_value = attrs.pop('image_width', None)
        image_width = 'style="width:{0}"'.format(image_width_value) \
            if width_value else ''
        attrs_string = flatatt(attrs)

        FLICKR_API_KEY = getattr(settings, 'FLICKR_API_KEY', None)
        GOOGLE_API_KEY = getattr(settings, 'GOOGLE_API_KEY', None)
        GOOGLE_SENGINE_ID = getattr(settings, 'GOOGLE_SENGINE_ID', None)
        MEDIA_FIELD_API = reverse('media_field_api')

        if not FLICKR_API_KEY or not GOOGLE_API_KEY or not GOOGLE_SENGINE_ID:
            raise ImproperlyConfigured('You need to provide FLICKR_API_KEY, \
                                       GOOGLE_API_KEY and GOOGLE_SENGINE_ID\
                                       in your settings.py')

        return format_html(widget_template,
                           attrs=unicode(attrs_string),
                           width=unicode(width),
                           image_width=unicode(image_width),
                           value=unicode(val),
                           name=unicode(name),
                           flickr_api=unicode(FLICKR_API_KEY),
                           google_api=unicode(GOOGLE_API_KEY),
                           google_sengine=unicode(GOOGLE_SENGINE_ID),
                           media_field_api=unicode(MEDIA_FIELD_API))


class MediaField(URLField):
    widget = MediaFieldWidget
